@@ -1,5 +1,5 @@
 /* This file is tc-arm.h
-   Copyright (C) 1994-2022 Free Software Foundation, Inc.
+   Copyright (C) 1994-2023 Free Software Foundation, Inc.
    Contributed by Richard Earnshaw (rwe@pegasus.esprit.ec.org)
 	Modified by David Taylor (dtaylor@armltd.co.uk)
 
@@ -28,7 +28,6 @@
 
 #define WORKING_DOT_WORD
 
-#define COFF_MAGIC 	ARMMAGIC
 #define TARGET_ARCH 	bfd_arch_arm
 
 #define DIFF_EXPR_OK
@@ -98,8 +97,8 @@ extern bool tc_start_label_without_colon (void);
 #define tc_frob_fake_label(S) arm_frob_label (S)
 
 #ifdef OBJ_ELF
-#define md_end arm_md_end
-extern void arm_md_end (void);
+#define md_finish arm_md_finish
+extern void arm_md_finish (void);
 bool arm_is_eabi (void);
 
 #define md_post_relax_hook		arm_md_post_relax ()
