@@ -1,5 +1,5 @@
 /* Plugin control for the GNU linker.
-   Copyright (C) 2010-2020 Free Software Foundation, Inc.
+   Copyright (C) 2010-2023 Free Software Foundation, Inc.
 
    This file is part of the GNU Binutils.
 
@@ -51,12 +51,14 @@
 #if !(defined(errno) || defined(_MSC_VER) && defined(_INC_ERRNO))
 extern int errno;
 #endif
-#if !defined (HAVE_DLFCN_H) && defined (HAVE_WINDOWS_H)
+#if defined (HAVE_DLFCN_H)
+#include <dlfcn.h>
+#elif defined (HAVE_WINDOWS_H)
 #include <windows.h>
 #endif
 
 /* Report plugin symbols.  */
-bfd_boolean report_plugin_symbols;
+bool report_plugin_symbols;
 
 /* The suffix to append to the name of the real (claimed) object file
    when generating a dummy BFD to hold the IR symbols sent from the
@@ -85,10 +87,11 @@ typedef struct plugin
   size_t n_args;
   /* The plugin's event handlers.  */
   ld_plugin_claim_file_handler claim_file_handler;
+  ld_plugin_claim_file_handler_v2 claim_file_handler_v2;
   ld_plugin_all_symbols_read_handler all_symbols_read_handler;
   ld_plugin_cleanup_handler cleanup_handler;
   /* TRUE if the cleanup handlers have been called.  */
-  bfd_boolean cleanup_done;
+  bool cleanup_done;
 } plugin_t;
 
 typedef struct view_buffer
@@ -102,11 +105,14 @@ typedef struct view_buffer
    pointer.  */
 typedef struct plugin_input_file
 {
+  /* The dummy BFD.  */
   bfd *abfd;
+  /* The original input BFD.  Non-NULL if it is an archive member.  */
+  bfd *ibfd;
   view_buffer_t view_buffer;
   char *name;
   int fd;
-  bfd_boolean use_mmap;
+  bool use_mmap;
   off_t offset;
   off_t filesize;
 } plugin_input_file_t;
@@ -130,7 +136,7 @@ static plugin_t *called_plugin = NULL;
 static const char *error_plugin = NULL;
 
 /* State of linker "notice" interface before we poked at it.  */
-static bfd_boolean orig_notice_all;
+static bool orig_notice_all;
 
 /* Original linker callbacks, and the plugin version.  */
 static const struct bfd_link_callbacks *orig_callbacks;
@@ -138,7 +144,7 @@ static struct bfd_link_callbacks plugin_callbacks;
 
 /* Set at all symbols read time, to avoid recursively offering the plugin
    its own newly-added input files and libs to claim.  */
-bfd_boolean no_more_claiming = FALSE;
+bool no_more_claiming = false;
 
 #if HAVE_MMAP && HAVE_GETPAGESIZE
 /* Page size used by mmap.  */
@@ -154,6 +160,7 @@ static const enum ld_plugin_tag tv_header_tags[] =
   LDPT_LINKER_OUTPUT,
   LDPT_OUTPUT_NAME,
   LDPT_REGISTER_CLAIM_FILE_HOOK,
+  LDPT_REGISTER_CLAIM_FILE_HOOK_V2,
   LDPT_REGISTER_ALL_SYMBOLS_READ_HOOK,
   LDPT_REGISTER_CLEANUP_HOOK,
   LDPT_ADD_SYMBOLS,
@@ -171,12 +178,12 @@ static const enum ld_plugin_tag tv_header_tags[] =
 static const size_t tv_header_size = ARRAY_SIZE (tv_header_tags);
 
 /* Forward references.  */
-static bfd_boolean plugin_notice (struct bfd_link_info *,
-				  struct bfd_link_hash_entry *,
-				  struct bfd_link_hash_entry *,
-				  bfd *, asection *, bfd_vma, flagword);
+static bool plugin_notice (struct bfd_link_info *,
+			   struct bfd_link_hash_entry *,
+			   struct bfd_link_hash_entry *,
+			   bfd *, asection *, bfd_vma, flagword);
 
-static bfd_cleanup plugin_object_p (bfd *);
+static bfd_cleanup plugin_object_p (bfd *, bool);
 
 #if !defined (HAVE_DLFCN_H) && defined (HAVE_WINDOWS_H)
 
@@ -220,7 +227,7 @@ set_plugin_error (const char *plugin)
 }
 
 /* Test if an error occurred.  */
-static bfd_boolean
+static bool
 plugin_error_p (void)
 {
   return error_plugin != NULL;
@@ -309,7 +316,7 @@ static bfd *
 plugin_get_ir_dummy_bfd (const char *name, bfd *srctemplate)
 {
   bfd *abfd;
-  bfd_boolean bfd_plugin_target;
+  bool bfd_plugin_target;
 
   bfd_use_reserved_id = 1;
   bfd_plugin_target = bfd_plugin_target_p (srctemplate->xvec);
@@ -343,7 +350,7 @@ plugin_get_ir_dummy_bfd (const char *name, bfd *srctemplate)
 }
 
 /* Check if the BFD passed in is an IR dummy object file.  */
-static inline bfd_boolean
+static inline bool
 is_ir_dummy_bfd (const bfd *abfd)
 {
   /* ABFD can sometimes legitimately be NULL, e.g. when called from one
@@ -415,7 +422,7 @@ asymbol_from_plugin_symbol (bfd *abfd, asymbol *asym,
 
   if (bfd_get_flavour (abfd) == bfd_target_elf_flavour)
     {
-      elf_symbol_type *elfsym = elf_symbol_from (abfd, asym);
+      elf_symbol_type *elfsym = elf_symbol_from (asym);
       unsigned char visibility;
 
       if (!elfsym)
@@ -462,6 +469,15 @@ register_claim_file (ld_plugin_claim_file_handler handler)
   return LDPS_OK;
 }
 
+/* Register a claim-file version 2 handler.  */
+static enum ld_plugin_status
+register_claim_file_v2 (ld_plugin_claim_file_handler_v2 handler)
+{
+  ASSERT (called_plugin);
+  called_plugin->claim_file_handler_v2 = handler;
+  return LDPS_OK;
+}
+
 /* Register an all-symbols-read handler.  */
 static enum ld_plugin_status
 register_all_symbols_read (ld_plugin_all_symbols_read_handler handler)
@@ -563,7 +579,7 @@ get_view (const void *handle, const void **viewp)
   buffer = mmap (NULL, size, PROT_READ, MAP_PRIVATE, input->fd, offset);
   if (buffer != MAP_FAILED)
     {
-      input->use_mmap = TRUE;
+      input->use_mmap = true;
 # if HAVE_GETPAGESIZE
       buffer += bias;
 # endif
@@ -573,7 +589,7 @@ get_view (const void *handle, const void **viewp)
     {
       char *p;
 
-      input->use_mmap = FALSE;
+      input->use_mmap = false;
 
       if (lseek (input->fd, offset, SEEK_SET) < 0)
 	return LDPS_ERR;
@@ -605,28 +621,36 @@ get_view (const void *handle, const void **viewp)
   return LDPS_OK;
 }
 
+/* Release plugin file descriptor.  */
+
+static void
+release_plugin_file_descriptor (plugin_input_file_t *input)
+{
+  if (input->fd != -1)
+    {
+      bfd_plugin_close_file_descriptor (input->ibfd, input->fd);
+      input->fd = -1;
+    }
+}
+
 /* Release the input file.  */
 static enum ld_plugin_status
 release_input_file (const void *handle)
 {
   plugin_input_file_t *input = (plugin_input_file_t *) handle;
   ASSERT (called_plugin);
-  if (input->fd != -1)
-    {
-      close (input->fd);
-      input->fd = -1;
-    }
+  release_plugin_file_descriptor (input);
   return LDPS_OK;
 }
 
 /* Return TRUE if a defined symbol might be reachable from outside the
    universe of claimed objects.  */
-static inline bfd_boolean
+static inline bool
 is_visible_from_outside (struct ld_plugin_symbol *lsym,
 			 struct bfd_link_hash_entry *blhe)
 {
   if (bfd_link_relocatable (&link_info))
-    return TRUE;
+    return true;
   if (blhe->non_ir_ref_dynamic
       || link_info.export_dynamic
       || bfd_link_dll (&link_info))
@@ -634,9 +658,9 @@ is_visible_from_outside (struct ld_plugin_symbol *lsym,
       /* Check if symbol is hidden by version script.  */
       if (bfd_hide_sym_by_version (link_info.version_info,
 				   blhe->root.string))
-	return FALSE;
+	return false;
       /* Only ELF symbols really have visibility.  */
-      if (bfd_get_flavour (link_info.output_bfd) == bfd_target_elf_flavour)
+      if (is_elf_hash_table (link_info.hash))
 	{
 	  struct elf_link_hash_entry *el = (struct elf_link_hash_entry *)blhe;
 	  int vis = ELF_ST_VISIBILITY (el->other);
@@ -656,7 +680,7 @@ is_visible_from_outside (struct ld_plugin_symbol *lsym,
 	      || lsym->visibility == LDPV_PROTECTED);
     }
 
-  return FALSE;
+  return false;
 }
 
 /* Return LTO kind string name that corresponds to IDX enum value.  */
@@ -743,7 +767,7 @@ get_symbols (const void *handle, int nsyms, struct ld_plugin_symbol *syms,
       int res;
       struct bfd_link_hash_entry *h
 	= bfd_link_hash_lookup (link_info.hash, syms[n].name,
-				FALSE, FALSE, TRUE);
+				false, false, true);
       enum { wrap_none, wrapper, wrapped } wrap_status = wrap_none;
 
       if (syms[n].def != LDPK_UNDEF && syms[n].def != LDPK_WEAKUNDEF)
@@ -762,7 +786,7 @@ get_symbols (const void *handle, int nsyms, struct ld_plugin_symbol *syms,
 	{
 	  blhe = bfd_wrapped_link_hash_lookup (link_info.output_bfd,
 					       &link_info, syms[n].name,
-					       FALSE, FALSE, TRUE);
+					       false, false, true);
 	  /* Check if a symbol is a wrapped symbol.  */
 	  if (blhe && blhe != h)
 	    wrap_status = wrapped;
@@ -852,8 +876,11 @@ get_symbols (const void *handle, int nsyms, struct ld_plugin_symbol *syms,
 	  /* We need to know if the sym is referenced from non-IR files.  Or
 	     even potentially-referenced, perhaps in a future final link if
 	     this is a partial one, perhaps dynamically at load-time if the
-	     symbol is externally visible.  Also check for wrapper symbol.  */
-	  if (blhe->non_ir_ref_regular || wrap_status == wrapper)
+	     symbol is externally visible.  Also check for __real_SYM
+	     reference and wrapper symbol.  */
+	  if (blhe->non_ir_ref_regular
+	      || blhe->ref_real
+	      || wrap_status == wrapper)
 	    res = LDPR_PREVAILING_DEF;
 	  else if (wrap_status == wrapped)
 	    res = LDPR_RESOLVED_IR;
@@ -922,7 +949,7 @@ static enum ld_plugin_status
 set_extra_library_path (const char *path)
 {
   ASSERT (called_plugin);
-  ldfile_add_library_path (xstrdup (path), FALSE);
+  ldfile_add_library_path (xstrdup (path), false);
   return LDPS_OK;
 }
 
@@ -936,14 +963,14 @@ message (int level, const char *format, ...)
   switch (level)
     {
     case LDPL_INFO:
-      vfinfo (stdout, format, args, FALSE);
+      vfinfo (stdout, format, args, false);
       putchar ('\n');
       break;
     case LDPL_WARNING:
       {
 	char *newfmt = concat (_("%P: warning: "), format, "\n",
 			       (const char *) NULL);
-	vfinfo (stdout, newfmt, args, TRUE);
+	vfinfo (stdout, newfmt, args, true);
 	free (newfmt);
       }
       break;
@@ -955,7 +982,7 @@ message (int level, const char *format, ...)
 			       _("%P: error: "), format, "\n",
 			       (const char *) NULL);
 	fflush (stdout);
-	vfinfo (stderr, newfmt, args, TRUE);
+	vfinfo (stderr, newfmt, args, true);
 	fflush (stderr);
 	free (newfmt);
       }
@@ -1003,6 +1030,9 @@ set_tv_header (struct ld_plugin_tv *tv)
 	case LDPT_REGISTER_CLAIM_FILE_HOOK:
 	  TVU(register_claim_file) = register_claim_file;
 	  break;
+	case LDPT_REGISTER_CLAIM_FILE_HOOK_V2:
+	  TVU(register_claim_file_v2) = register_claim_file_v2;
+	  break;
 	case LDPT_REGISTER_ALL_SYMBOLS_READ_HOOK:
 	  TVU(register_all_symbols_read) = register_all_symbols_read;
 	  break;
@@ -1115,8 +1145,8 @@ plugin_load_plugins (void)
   orig_callbacks = link_info.callbacks;
   plugin_callbacks = *orig_callbacks;
   plugin_callbacks.notice = &plugin_notice;
-  link_info.notice_all = TRUE;
-  link_info.lto_plugin_active = TRUE;
+  link_info.notice_all = true;
+  link_info.lto_plugin_active = true;
   link_info.callbacks = &plugin_callbacks;
 
   register_ld_plugin_object_p (plugin_object_p);
@@ -1128,10 +1158,11 @@ plugin_load_plugins (void)
 
 /* Call 'claim file' hook for all plugins.  */
 static int
-plugin_call_claim_file (const struct ld_plugin_input_file *file, int *claimed)
+plugin_call_claim_file (const struct ld_plugin_input_file *file, int *claimed,
+			bool known_used)
 {
   plugin_t *curplug = plugins_list;
-  *claimed = FALSE;
+  *claimed = false;
   while (curplug && !*claimed)
     {
       if (curplug->claim_file_handler)
@@ -1139,7 +1170,10 @@ plugin_call_claim_file (const struct ld_plugin_input_file *file, int *claimed)
 	  enum ld_plugin_status rv;
 
 	  called_plugin = curplug;
-	  rv = (*curplug->claim_file_handler) (file, claimed);
+	  if (curplug->claim_file_handler_v2)
+	    rv = (*curplug->claim_file_handler_v2) (file, claimed, known_used);
+	  else
+	    rv = (*curplug->claim_file_handler) (file, claimed);
 	  called_plugin = NULL;
 	  if (rv != LDPS_OK)
 	    set_plugin_error (curplug->name);
@@ -1171,7 +1205,7 @@ plugin_cleanup (bfd *abfd ATTRIBUTE_UNUSED)
 }
 
 static bfd_cleanup
-plugin_object_p (bfd *ibfd)
+plugin_object_p (bfd *ibfd, bool known_used)
 {
   int claimed;
   plugin_input_file_t *input;
@@ -1211,22 +1245,24 @@ plugin_object_p (bfd *ibfd)
 
   file.handle = input;
   input->abfd = abfd;
+  input->ibfd = ibfd->my_archive != NULL ? ibfd : NULL;
   input->view_buffer.addr = NULL;
   input->view_buffer.filesize = 0;
   input->view_buffer.offset = 0;
   input->fd = file.fd;
-  input->use_mmap = FALSE;
+  input->use_mmap = false;
   input->offset = file.offset;
   input->filesize = file.filesize;
   input->name = plugin_strdup (abfd, bfd_get_filename (ibfd));
 
   claimed = 0;
 
-  if (plugin_call_claim_file (&file, &claimed))
+  if (plugin_call_claim_file (&file, &claimed, known_used))
     einfo (_("%F%P: %s: plugin reported error claiming file\n"),
 	   plugin_error_plugin ());
 
-  if (input->fd != -1 && !bfd_plugin_target_p (ibfd->xvec))
+  if (input->fd != -1
+      && (!claimed || !bfd_plugin_target_p (ibfd->xvec)))
     {
       /* FIXME: fd belongs to us, not the plugin.  GCC plugin, which
 	 doesn't need fd after plugin_call_claim_file, doesn't use
@@ -1236,8 +1272,7 @@ plugin_object_p (bfd *ibfd)
 	 release_input_file after it is done, uses BFD plugin target
 	 vector.  This scheme doesn't work when a plugin needs fd and
 	 doesn't use BFD plugin target vector neither.  */
-      close (input->fd);
-      input->fd = -1;
+      release_plugin_file_descriptor (input);
     }
 
   if (claimed)
@@ -1277,7 +1312,7 @@ void
 plugin_maybe_claim (lang_input_statement_type *entry)
 {
   ASSERT (entry->header.type == lang_input_statement_enum);
-  if (plugin_object_p (entry->the_bfd))
+  if (plugin_object_p (entry->the_bfd, true))
     {
       bfd *abfd = entry->the_bfd->plugin_dummy_bfd;
 
@@ -1301,7 +1336,7 @@ plugin_call_all_symbols_read (void)
   plugin_t *curplug = plugins_list;
 
   /* Disable any further file-claiming.  */
-  no_more_claiming = TRUE;
+  no_more_claiming = true;
 
   while (curplug)
     {
@@ -1329,7 +1364,7 @@ plugin_call_cleanup (void)
       if (curplug->cleanup_handler && !curplug->cleanup_done)
 	{
 	  enum ld_plugin_status rv;
-	  curplug->cleanup_done = TRUE;
+	  curplug->cleanup_done = true;
 	  called_plugin = curplug;
 	  rv = (*curplug->cleanup_handler) ();
 	  called_plugin = NULL;
@@ -1349,7 +1384,7 @@ plugin_call_cleanup (void)
    non_ir_ref_dynamic as appropriate.  We have to notice_all symbols,
    because we won't necessarily know until later which ones will be
    contributed by IR files.  */
-static bfd_boolean
+static bool
 plugin_notice (struct bfd_link_info *info,
 	       struct bfd_link_hash_entry *h,
 	       struct bfd_link_hash_entry *inh,
@@ -1363,7 +1398,7 @@ plugin_notice (struct bfd_link_info *info,
   if (h != NULL)
     {
       bfd *sym_bfd;
-      bfd_boolean ref = FALSE;
+      bool ref = false;
 
       if (h->type == bfd_link_hash_warning)
 	h = h->u.i.link;
@@ -1383,13 +1418,13 @@ plugin_notice (struct bfd_link_info *info,
 	      || inh->type == bfd_link_hash_new)
 	    {
 	      if ((abfd->flags & DYNAMIC) == 0)
-		inh->non_ir_ref_regular = TRUE;
+		inh->non_ir_ref_regular = true;
 	      else
-		inh->non_ir_ref_dynamic = TRUE;
+		inh->non_ir_ref_dynamic = true;
 	    }
 
 	  if (h->type != bfd_link_hash_new)
-	    ref = TRUE;
+	    ref = true;
 	}
 
       /* Nothing to do here for warning symbols.  */
@@ -1409,7 +1444,7 @@ plugin_notice (struct bfd_link_info *info,
 	       && (h->u.undef.abfd == NULL
 		   || (h->u.undef.abfd->flags & BFD_PLUGIN) != 0))
 	     h->u.undef.abfd = abfd;
-	  ref = TRUE;
+	  ref = true;
 	}
 
 
@@ -1425,7 +1460,7 @@ plugin_notice (struct bfd_link_info *info,
 	      h->type = bfd_link_hash_undefweak;
 	      h->u.undef.abfd = sym_bfd;
 	    }
-	  ref = TRUE;
+	  ref = true;
 	}
 
       /* Otherwise, it must be a new def.
@@ -1436,8 +1471,10 @@ plugin_notice (struct bfd_link_info *info,
 	 this by making the symbol appear to be undefined.
 
 	 NB: We change the previous definition in the IR object to
-	 undefweak only after all LTO symbols have been read.  */
-      else if (info->lto_all_symbols_read
+	 undefweak only after all LTO symbols have been read or for
+	 non-ELF targets.  */
+      else if ((info->lto_all_symbols_read
+		|| bfd_get_flavour (abfd) != bfd_target_elf_flavour)
 	       && (((h->type == bfd_link_hash_defweak
 		     || h->type == bfd_link_hash_defined)
 		    && is_ir_dummy_bfd (sym_bfd = h->u.def.section->owner))
@@ -1451,9 +1488,9 @@ plugin_notice (struct bfd_link_info *info,
       if (ref)
 	{
 	  if ((abfd->flags & DYNAMIC) == 0)
-	    h->non_ir_ref_regular = TRUE;
+	    h->non_ir_ref_regular = true;
 	  else
-	    h->non_ir_ref_dynamic = TRUE;
+	    h->non_ir_ref_dynamic = true;
 	}
     }
 
@@ -1462,9 +1499,9 @@ plugin_notice (struct bfd_link_info *info,
       || orig_notice_all
       || (info->notice_hash != NULL
 	  && bfd_hash_lookup (info->notice_hash, orig_h->root.string,
-			      FALSE, FALSE) != NULL))
+			      false, false) != NULL))
     return (*orig_callbacks->notice) (info, orig_h, inh,
 				      abfd, section, value, flags);
-  return TRUE;
+  return true;
 }
 #endif /* BFD_SUPPORTS_PLUGINS */
