@@ -1,6 +1,6 @@
 /* Disassemble Xilinx microblaze instructions.
 
-   Copyright (C) 2009-2018 Free Software Foundation, Inc.
+   Copyright (C) 2009-2023 Free Software Foundation, Inc.
 
    This file is part of the GNU opcodes library.
 
@@ -28,10 +28,10 @@ extern "C" {
 
 extern enum microblaze_instr microblaze_decode_insn (long, int *, int *,
 						     int *, int *);
-extern unsigned long microblaze_get_target_address (long, bfd_boolean, int,
-			       long, long, long, bfd_boolean *, bfd_boolean *);
-
-extern enum microblaze_instr get_insn_microblaze (long, bfd_boolean *,
+extern unsigned long microblaze_get_target_address (long, bool, int,
+						    long, long, long,
+						    bool *, bool *);
+extern enum microblaze_instr get_insn_microblaze (long, bool *,
 						  enum microblaze_instr_type *,
   		     				  short *);
 
